'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
/* eslint-disable max-params */

/**
 * 开始渲染核心路由组件，继承方式
 * index > kernal-component -> lifecycle -> common -> base
 */

const React = require('react');

const award_utils_1 = require('award-utils');

const client_1 = require('award-plugin/client');

const kernal_component_1 = require('./kernal-component');

const routerLifecycle_1 = require('../utils/routerLifecycle');

exports.default = (routes, match_routes, INITIAL_STATE, Component) => {
  /**
   * 入口组件渲染，处理react生命周期
   */
  return class AwardRouter extends kernal_component_1.default {
    constructor(props, context) {
      super(props, context, [Component, routes, match_routes, INITIAL_STATE]);
      let errorInfo = null; // webpack会删除该段代码

      if (process.env.NODE_ENV === 'development') {
        if (window.award_hmr && window.award_hmr_error) {
          // 热更新
          errorInfo = window.award_hmr_error;
        }
      } // 正常渲染，且发生的错误是路由外的才录用

      if (!errorInfo && INITIAL_STATE.AwardException && !INITIAL_STATE.AwardException.routerError) {
        errorInfo = INITIAL_STATE.AwardException;
      }

      this.state = {
        errorInfo,
        loading: null,
        data: INITIAL_STATE.award
      };
    }

    shouldComponentUpdate(nextProps, nextState) {
      if (process.env.NODE_ENV === 'development') {
        if (window.award_hmr) {
          return true;
        }
      }

      if (
        this.state.loading !== nextState.loading ||
        this.state.errorInfo !== nextState.errorInfo
      ) {
        return true;
      }

      return false;
    }

    async componentDidMount() {
      let pass = true;

      if (process.env.NODE_ENV === 'development') {
        if (window.award_hmr) {
          this.emitter.off('routeDidUpdate');
          this.emitter.off('routerDidUpdate');
          pass = false;
        }
      }

      if (pass) {
        const { firstRender, ssr } = award_utils_1.loadParams.get();

        if (firstRender && !ssr && this.history) {
          // 首次渲染结束触发,触发重定向
          this.emitter.emit('routerWillUpdate_!ssr', this.history);
        }
      } // 监听单个路由的切换

      this.emitter.on('routeDidUpdate', (info) => {
        if (pass) {
          const { fn, data } = info;
          const to = this.target;
          const from = this.lastTarget;
          fn(to, from, data);
        } else {
          pass = true;
        }
      }); // 监听全局路由的切换

      this.emitter.on('routerDidUpdate', (data) => {
        if (pass) {
          routerLifecycle_1.routerDidUpdate({
            to: this.target,
            from: this.lastTarget,
            data
          });
        } else {
          pass = true;
        }
      }); // 如果发生错误了，将立刻触发routerDidUpdate

      if (
        INITIAL_STATE === null || INITIAL_STATE === void 0 ? void 0 : INITIAL_STATE.AwardException
      ) {
        this.emitter.emit('routerDidUpdate', INITIAL_STATE);
      }
    }

    async componentDidCatch(error) {
      client_1.default.hooks.catchError({
        type: 'global',
        error
      });
      const message = error.message ? error.message : null;
      const stack = error.stack ? error.stack : null;
      const errorInfo = await award_utils_1.Exception.handleError.call(
        null,
        {
          message,
          stack,
          routerError: false
        },
        (component) => {
          this.setState({
            loading: component
          });
        }
      );

      if (process.env.NODE_ENV === 'development') {
        window.award_hmr_error = errorInfo;
      }

      this.setState({
        errorInfo,
        loading: null
      });
    }

    render() {
      if (this.state.loading) {
        const { loading } = this.state;

        if (React.isValidElement(loading)) {
          return React.cloneElement(
            loading,
            typeof loading.type === 'string'
              ? {}
              : {
                  data: INITIAL_STATE,
                  routerError: false
                }
          );
        }

        if (typeof loading === 'function') {
          return React.createElement(loading, {
            data: INITIAL_STATE,
            routerError: false
          });
        }

        return null;
      }

      if (this.state.errorInfo) {
        const ErrorComponent = award_utils_1.Exception.shot();

        const redirect = (url) => {
          if (!/^http(s)?:/.test(url)) {
            const { basename } = award_utils_1.loadParams.get();
            window.location.href = basename + url;
          } else {
            window.location.href = url;
          }
        };

        return React.createElement(
          ErrorComponent,
          Object.assign({}, this.state.errorInfo, {
            redirect: redirect
          })
        );
      }

      const Provider = this.provider();
      const Router = this.router();
      const Root = this.root();
      const Prompt = this.prompt();
      return React.createElement(
        Provider,
        null,
        React.createElement(
          Router,
          null,
          React.createElement(Prompt, null),
          React.createElement(Root, null, React.createElement(Component, null))
        )
      );
    }
  };
};
