'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const React = require('react');

const AwardContext_1 = require('award-utils/AwardContext');

const setAward_1 = require('award-utils/setAward');

const setAward_2 = require('../../setAward');

const removeAward_1 = require('../../removeAward');

let hmrAwardValue = {};

const Abort = (RootComponent) =>
  class AwardAbort extends React.Component {
    shouldComponentUpdate() {
      return false;
    }

    render() {
      return React.createElement(RootComponent, null);
    }
  };
/**
 * 创建两个context, 后续combineContextType
 * @param AwardContext this.state.award
 */

exports.default = ({ Component, INITIAL_STATE }) => {
  // Provider层禁用渲染
  const AbortComponent = Abort(Component);
  return class Award extends React.Component {
    constructor(props) {
      super(props);
      let pass = true;

      if (process.env.NODE_ENV === 'development') {
        if (window.award_hmr) {
          pass = false;
        }
      }

      if (pass) {
        hmrAwardValue = INITIAL_STATE.award;
      }

      this.state = {
        award: hmrAwardValue
      };

      if (process.env.RUN_ENV === 'web') {
        if (process.env.NODE_ENV === 'development') {
          if (window.award_hmr) {
            setAward_2.clean();
            removeAward_1.clean();
          }
        }

        this.setAward = this.setAward.bind(this);
        setAward_1.default.set(this.setAward);
        setAward_2.default(this.setAward);
        removeAward_1.default(this.removeAward.bind(this));
      }
    }

    componentDidMount() {
      if (process.env.NODE_ENV === 'development') {
        if (window.award_hmr) {
          console.info('移除award_hmr');
          delete window.award_hmr;
        }
      }
    }
    /**
     * this.setAward({'name':2})
     */

    setAward(obj = {}) {
      if (obj.constructor === Object && Object.keys(obj).length) {
        hmrAwardValue = { ...this.state.award, ...obj }; // 处理异步更新状态

        return new Promise((resolve) => {
          this.setState(
            {
              award: hmrAwardValue
            },
            () => {
              setTimeout(resolve, 0);
            }
          );
        });
      } else {
        console.warn('setAward必须接受一个不为空的对象');
      }
    }
    /**
     * this.removeAward('name')
     */

    removeAward(key) {
      if (this.state.award[key]) {
        delete this.state.award[key];
        this.setState({
          award: { ...this.state.award }
        });
      } else {
        console.warn(`removeAward移除的【${key}】不存在`);
      }
    }

    render() {
      return React.createElement(
        AwardContext_1.default.Provider,
        {
          value: this.state.award
        },
        React.createElement(AbortComponent, null)
      );
    }
  };
};
